#ifndef GRAPHICS_FONT_H
#define GRAPHICS_FONT_H TRUE

/*
**  font.h
**
**  (C) Copyright 1998-2003 Rocklyte Systems.
**      All Rights Reserved.
*/

#ifndef MAIN_H
#include <pandora/main.h>
#endif

/*****************************************************************************
** Font class.
*/

#define VER_FONT  2.0

#define SIZE_FONTFACE     30
#define SIZE_FONTSTYLE    40
#define SIZE_FONTBUFFER   160
#define SIZE_FONTLOCATION 160

struct FontCharacter {
   WORD Width;
   WORD Advance;
   UWORD Offset;
   UWORD OutlineOffset;
};

struct Font {
   OBJECT_HEADER              /* Class header */
   struct Bitmap *Bitmap;     /* Pointer to destination Bitmap */
   struct RGB *OutlineRGB;    /* Outline colour */
   struct RGB *UnderlineRGB;  /* Underline colour */
   struct RGB *ColourRGB;     /* Colour */
   STRING String;             /* String for drawing etc */
   LONG   Flags;              /* Special flags */
   LONG   Point;              /* The Point/Height of the font (arbitrary size, non-exact) */
   LONG   Gutter;             /* "External leading" in pixels (fixed fonts only) */
   LONG   GlyphSpacing;       /* Amount of spacing between characters (additional to normal horizontal spacing) */
   LONG   LineSpacing;        /* Vertical spacing between lines */
   LONG   LineCount;          /* Automatically updated when the Font->String is set */
   LONG   XCoord;             /* X Coordinate */
   LONG   YCoord;             /* Y Coordinate */
   LONG   TabSize;            /* Tab differential */
   LONG   TotalCharacters;    /* Total number of characters / glyphs supported by the font (V2.0) */
   LONG   WrapWidth;          /* Width at which word-wrap is encountered (V2.0) */
   LONG   FixedWidth;         /* Force fixed width (V2.0) */
   STRING Location;           /* The location of the font file as derived from the name of the face (V2.0).  Field may be set directly if the font source is unregistered */
   LONG   Height;             /* The point size of the font (vertical bearing), expressed in pixels */
   STRING Style;              /* The font style (Regular, Bold, Italic, Condensed, Light etc) */
   LONG   Leading;            /* "Internal leading" in pixels (fixed fonts only) */
   LONG   MaxHeight;          /* Maximum possible pixel height per character, covering the entire character set at the chosen point size */
   DOUBLE Angle;              /* String transformation angle */
   STRING Face;               /* The face of the font */
   LONG   AlignFlags;
   LONG   AlignWidth;
   LONG   AlignHeight;
   WORD   *Tabs;
   LONG   TabWidth;           /* The width of each individual tab character */

   PRIVATE_FIELDS
   struct RGB prvColourRGB;
   struct RGB prvUnderlineRGB;
   struct RGB prvOutlineRGB;
   UBYTE *prvData;
   UBYTE *prvOutline;
   LONG prvStrWidth;
   WORD prvTranslucency;
   WORD prvSpaceWidth;          /* Pixel width of word breaks */
   WORD prvBitmapHeight;
   #ifdef FT_FREETYPE_H
      FT_Face FTFace;
   #else
      LONG FTFace;
   #endif
   UBYTE prvFace[SIZE_FONTFACE];
   UBYTE prvBuffer[SIZE_FONTBUFFER];
   UBYTE prvLocation[SIZE_FONTLOCATION];
   UBYTE prvStyle[SIZE_FONTSTYLE];
   struct FontCharacter prvChar[256];
   UBYTE prvDefaultChar;
};

/*****************************************************************************
** Flags
*/

#define FTF_PREFERSCALED 0x00000001  /* A scaled font is required on initialisation */
#define FTF_PREFERFIXED  0x00000002  /* A fixed font (monochrome, no transforms) is required on initialisation */
#define FTF_ANTIALIAS    0x00000004  /* Smooth the edges */
#define FTF_WORDWRAP     0x00000008  /* Word wrap */
#define FTF_UNDERLINE    0x00000010  /* Underline the font */
#define FTF_HEAVYLINE    0x00000020  /* Underline the font with a double-sized line */
#define FTF_UPDATEXY     0x00000100  /* Update the XCoord and YCoord with each draw */
#define FTF_OUTLINE      0x00000200  /* Draw an outline around the font */
#define FTF_QUICKALIAS   0x00000400  /* Quick anti-aliasing is useful for games */
#define FTF_ABSOLUTEXY   0x00000800  /* Use of absolute X/Y coordinates guarantees that no pixels will be drawn to the left or above the root drawing coordinate */
#define FTF_CHARCLIP     0x00001000  /* Clip words by adding dots to the end of the string */

#define FTF_KERNING      0x80000000  /* Font has kerning information */
#define FTF_ITALICS      0x40000000  /* Font is described as using italics */
#define FTF_BOLD         0x20000000  /* Font is described as having a bold weight */
#define FTF_SCALABLE     0x10000000  /* This is set if the font is scalable, otherwise assume fixed */

#define FTF_ITALIC       FTF_ITALICS
#define FTF_SMOOTH       FTF_ANTIALIAS
#define FTF_DIRECTXY     FTF_ABSOLUTEXY
#define FTF_NEEDSCALED   FTF_PREFERSCALED
#define FTF_NEEDFIXED    FTF_PREFERFIXED

/*****************************************************************************
** Methods
*/

#define MT_CalcStringWidth   -1
#define MT_ConvertFontCoords -2 /* Version 2.0 */

struct mtCalcStringWidth {
   STRING String;
   LONG Length;
   LONG Width;
};

struct mtConvertFontCoords {
   STRING String;
   LONG XCoord;
   LONG YCoord;
   LONG Column;
   LONG Row;
   LONG ByteColumn;
   LONG BytePosition;
};

/*****************************************************************************
** FontServer class
*/

#define VER_FONTSERVER 1.0

struct FontServer {
   OBJECT_HEADER              /* Class header */

   PRIVATE_FIELDS
   OBJECTID prvConfigID;
};

/*****************************************************************************
** FontServer methods
*/

#define MT_InstallFont -1
#define MT_RemoveFont  -2
#define MT_SelectFont  -3

struct mtInstallFont {
   STRING Files;
};

struct mtRemoveFont {
   STRING Name;
};

struct mtSelectFont {
   STRING Name;
   STRING Style;
   LONG   Point;
   LONG   Flags;
   STRING Location; /* Result */
   LONG   Handle;   /* Result */
};

#endif /* GRAPHICS_FONT_H */
