#ifndef  MODULES_KERNEL_H
#define  MODULES_KERNEL_H

/*
**  kernel.h
**
**  (C) Copyright 1996-2004 Rocklyte Systems.
**      All Rights Reserved.
*/

#ifndef  MAIN_H
#include <pandora/main.h>
#endif

#define STR_MATCHCASE   0x0001 /* Case comparison */
#define STR_MATCHLENGTH 0x0002 /* Comparison only OK if lengths match */
#define STR_WILDCARD    0x0004 /* Allow for wildcards '*' and '?' */
#define STR_CASE        STR_MATCHCASE
#define STR_WILDCARDS   STR_WILDCARD

enum {
   RES_MESSAGEQUEUE=1,
   RES_TIMEOUT,
   RES_MESSAGEHANDLER,
   RES_KERNELVERSION,
   RES_KERNELREVISION,
   RES_VIDEORECOVERY,
   RES_CONSOLEFD,
   RES_ALLOCMEMLIMIT,
   RES_FREESWAP,
   RES_GLOBALINSTANCE,
   RES_INSTANCE,
   RES_CURRENTTASK,
   RES_SHARECONTROL,
   RES_USERID,
   RES_CURRENTTASKPTR,
   RES_TOTALMEM,
   RES_FREEMEM,
   RES_SHAREDMEM,
   RES_TOTALSWAP,
   RES_FDS,
   RES_TOTALFDS,
   RES_TESTCODE,
   RES_PROCESSID,
   RES_X11FD,
   RES_PLATFORM,
   RES_PRIVILEGEDUSER,
   RES_PRIVILEGED,
   RES_KEYBOARDFD,
   RES_ERRORCODES,
   RES_ERRORCODES_TOTAL,
   RES_HEADERCODES,
   RES_HEADERCODES_TOTAL
};

struct FieldSetting {
   LONG FieldID;
   LONG FieldType;
   APTR Value;
};

struct ClassMethods {
   WORD   MethodID;      /* Method ID */
   WORD   StructureSize; /* Size of the argument structure (NULL if no arguments) */
   STRING Name;          /* Name of the method */
   STRING Arguments;     /* Argument description string */
};

struct SystemActions {
  WORD ActionID;
  WORD StructureSize;
  STRING Name;
  STRING Arguments;
};

/*** Flags for NotifySubscribers ***/

#define NSF_EXCLUSIVE  0x00000001
#define NSF_LOCALTASK  0x00000002
#define NSF_OTHERTASKS 0x00000004
#define NSF_FORCEDELAY 0x00000008

/*** Flags for semaphores ***/

#define SMF_NOBLOCKING  0x00000001
#define SMF_EXISTS      0x00000002
#define SMF_NONBLOCKING SMF_NOBLOCKING

/*** Struture for MemoryPtrInfo and MemoryIDInfo ***/

#define VER_MemoryInfo 2

struct MemInfo {
   APTR     Start;       /* The starting address of the memory block */
   OBJECTID ObjectID;    /* The object that owns the memory block */
   LONG     Size;        /* The size of the memory block */
   WORD     AccessCount; /* Amount of accesses/locks on this block */
   WORD     Flags;       /* The type of memory */
   MEMORYID MemoryID;    /* The unique ID of the memory block */
   OBJECTID LockID;      /* Task that currently has a lock on the block */
   OBJECTID TaskID;      /* The Task that owns the memory block */
   LONG     Handle;      /* [V2] Native system handle (e.g. the shmid in Linux) */
};

/*** Structure for ActionList */

struct ActionTable {
   STRING Name;
   struct FunctionField *Args;
   LONG Size;
};

/*** Structure for ListChildren ***/

struct ChildEntry {
   OBJECTID ObjectID;
   LONG ClassID;
};

/*** Structure for ListMemory ***/

struct ListMemory {
   MEMORYID MemoryID;    /* Pandora memory ID */
   LONG     Offset;      /* Byte offset if block is part of a memory pool */
   OBJECTID ObjectID;    /* Object that the address belongs to */
   APTR     Address;     /* Memory address, if the block is private to the caller */
   LONG     Size;        /* Size of the memory block */
   WORD     AccessCount; /* Count of locks on this address */
   WORD     Flags;       /* Memory flags */
   OBJECTID LockID;      /* Refers to the Task that has a current lock on the block */
   OBJECTID TaskID;
   LONG     AccessTime;
};

/*** Structure for ListTasks ***/

struct ListTasks {
   LONG     ProcessID;         /* Kernel process ID */
   OBJECTID TaskID;            /* Task ID for this array entry */
   LONG     WaitingProcessID;  /* If the task is waiting, this field reflects the other task's ID */
   MEMORYID WaitingMemoryID;   /* If the task is waiting, this field reflects the memory ID */
   LONG     WaitingTime;       /* If the task is waiting, the time at which the sleep started (msec) */
   MEMORYID MessageID;         /* Message queue ID */
   OBJECTID OutputID;          /* The object that the task should output information to */
   LONG     Semaphore;         /* The semaphore to signal when waking up the task (win32) */
   LONG     InstanceID;        /* Instance that the task belongs to */
   LONG     TotalMemoryLocks;
   struct   {
      MEMORYID MemoryID;
      WORD     Locks;
   } *MemoryLocks;
};

/*** Structure for ProcessMessages() ***/

struct NextMsg {
   struct Message Message;
   LONG Count;
};

/*** Structure for AlterMemory() ***/

struct AlterMemory {
   LONG Flags;
   MEMORYID MemoryID;
   OBJECTID TaskID;
   OBJECTID ObjectID;
   LONG Size;
};

struct FDTable {
   LONG FD;
   void (*Routine)(LONG, APTR);
   APTR Data;
};

/*** Structure for ListObjects() ***/

struct ObjectInfo {
   struct ObjectInfo *Next;     /* Next list entry */
   OBJECTPTR Object;            /* Pointer the object address (if in private memory) */
   OBJECTID  ObjectID;          /* The object's ID */
   OBJECTID  OwnerID;           /* The owner of the object */
   MEMORYID  MessageMID;        /* Message queue that is used for communication with the object */
   CLASSID   ClassID;           /* Class ID of the object */
   UBYTE     Name[MAX_NAMELEN]; /* Name of the object */
};

#define LOF_PUBLIC  0x00000001
#define LOF_PRIVATE 0x00000002
#define LOF_NAMES   0x00000004

#define ALF_MEMORYID  0x00000001
#define ALF_UNTRACKED 0x00000002
#define ALF_PUBLIC    0x00000004
#define ALF_PRIVATE   0x00000008

/*** Types for AllocateID() ***/

#define IDTYPE_MESSAGE  1
#define IDTYPE_GLOBAL   2

/****************************************************************************/

#ifdef LOCAL_KERNEL
ERROR     AccessMemory(MEMORYID, LONG, APTR *);
ERROR     AccessObject(OBJECTID, LONG, OBJECTPTR *);
ERROR     AccessSemaphore(LONG, LONG, LONG);
ERROR     Action(ACTIONID, APTR, APTR);
struct ActionTable * ActionList(void);
ERROR     ActionMsg(ACTIONID, OBJECTID, APTR, MEMORYID, CLASSID);
ERROR     ActionTags(ACTIONID, OBJECTPTR, ...);
ERROR     AddMsgHandler(LONG (*Handler)(LONG, LONG, APTR, LONG));
LONG      AllocateID(LONG);
ERROR     AllocateClassID(STRING Name, CLASSID *);
ERROR     AllocMemory(LONG, LONG, APTR *, MEMORYID *);
ERROR     AllocSemaphore(STRING, LONG, LONG, LONG *);
ERROR     AlterMemory(MEMORYID, struct AlterMemory *);
ERROR     CheckAction(OBJECTPTR, ACTIONID);
ERROR     CheckMemoryExists(MEMORYID);
ERROR     CheckObjectExists(OBJECTID, STRING);
ERROR     ClearMemory(APTR, LONG);
ERROR     CloneMemory(APTR, LONG, APTR *, MEMORYID *);
ERROR     CopyMemory(APTR, APTR, LONG);
ERROR     CreateObject(CLASSID, LONG, OBJECTPTR *, OBJECTID *, LONG, ...);
APTR      CurrentContext(void);
LONG      DebugState(LONG, LONG);
void      DeregisterFD(LONG);
void      DPrintF(STRING, STRING, ...);
ERROR     FastFindObject(STRING, CLASSID, OBJECTID *, LONG, LONG *);
OBJECTPTR FindClass(CLASSID, OBJECTPTR);
ERROR     FindObject(STRING, CLASSID, OBJECTID **, LONG *);
ERROR     FindPrivateObject(STRING, OBJECTPTR *);
ERROR     Forbid(void);
ERROR     FreeMemory(APTR);
ERROR     FreeMemoryID(MEMORYID);
ERROR     FreeSemaphore(LONG);
CLASSID   GetClassID(OBJECTID);
OBJECTID  GetOwnerID(OBJECTID);
ERROR     GetField(OBJECTPTR, LONG, LONG, APTR);
ERROR     GetFields(OBJECTPTR, ...);
ERROR     GetFieldVariable(OBJECTPTR Object, STRING FieldName, STRING Buffer, LONG);
ERROR     GetMessage(MEMORYID MessageID, LONG Type, LONG Flags, APTR Buffer, LONG);
ERROR     GetName(OBJECTPTR, STRING Buffer, LONG Length);
LARGE     GetResource(LONG ResourceID);
ERROR     IdentifyData(STRING Location, CLASSID *ClassID, CLASSID *SubClassID);
ERROR     ListChildren(OBJECTID, struct ChildEntry *, LONG *Count);
ERROR     ListMemory(LONG Flags, struct ListMemory **);
ERROR     ListObjects(CLASSID, LONG, struct ObjectInfo **);
ERROR     ListTasks(LONG Flags, struct ListTasks **);
ERROR     ManageAction(ACTIONID, APTR Routine);
ERROR     MemoryIDInfo(MEMORYID, LONG Version, struct MemInfo *);
ERROR     MemoryPtrInfo(APTR Address, LONG Version, struct MemInfo *);
ERROR     MessageHook(APTR Routine, LONG State);
ERROR     NewClass(STRING ClassName, OBJECTID *);
ERROR     NewObject(CLASSID, LONG Flags, OBJECTPTR *, OBJECTID *, STRING);
void      NotifySubscribers(OBJECTPTR, ACTIONID, APTR Parameters, LONG Flags);
ERROR     ObjectError(OBJECTPTR, LONG, ERROR);
ERROR     ObtainMethod(OBJECTPTR, ACTIONID, void **Result);
ERROR     Permit(void);
void      PrintDiagnosis(LONG, LONG);
ERROR     ProcessMessages(struct NextMsg *, LONG);
LONG      RandomNumber(LONG Range);
ERROR     ReallocMemory(APTR Memory, LONG NewSize, APTR *Result, MEMORYID *);
ERROR     RegisterFD(LONG, APTR, APTR);
MEMORYID  ReleaseMemory(APTR);
ERROR     ReleaseObject(OBJECTPTR);
ERROR     ReleaseSemaphore(LONG, LONG);
ERROR     RemoveMsgHandler(LONG (*Handler)(LONG, LONG, APTR, LONG));
LONG      ResolveClassName(STRING);
ERROR     ResolveFields(STRING, ...);
ERROR     ScanMessages(APTR, LONG *, LONG, APTR, LONG);
void      SelfDestruct(void);
ERROR     SendError(LONG, LONG);
ERROR     SetOwner(OBJECTPTR, OBJECTPTR);
APTR      SetContext(APTR);
ERROR     SetField(OBJECTPTR, LONG FieldID, LONG FieldType, APTR Data);
ERROR     SetFields(OBJECTPTR, ...);
ERROR     SetFieldVariable(OBJECTPTR, STRING FieldName, STRING Value);
ERROR     SetName(OBJECTPTR, STRING);
LARGE     SetResource(LONG, LARGE);
void      StepBack(void);
ERROR     SendMessage(MEMORYID MessageID, LONG Type, LONG Flags, APTR Data, LONG Size);
ERROR     StrCompare(STRING, STRING, LONG, LONG);
ERROR     SubscribeAction(OBJECTPTR, ACTIONID, OBJECTPTR);
ERROR     SubscribeActionTags(OBJECTPTR, OBJECTPTR, ACTIONID, ...);
ERROR     SubscribeChannel(OBJECTPTR, OBJECTPTR);
ERROR     SubscribeField(OBJECTPTR, FIELD, OBJECTPTR Subscriber);
ERROR     SubscribeTimer(OBJECTID, LONG);
LARGE     SystemTime(void);
LONG      TotalChildren(OBJECTID);
ERROR     UnsubscribeAction(OBJECTPTR, ACTIONID, OBJECTID SubscriberID);
ERROR     UnsubscribeChannel(OBJECTPTR, OBJECTID);
ERROR     UnsubscribeField(OBJECTPTR, FIELD, OBJECTID SubscriberID);
ERROR     UnsubscribeTimer(OBJECTID);
ERROR     UpdateMessage(APTR Queue, LONG MessageID, LONG Type, APTR Buffer, LONG BufferSize);
void      WaitTime(LONG Seconds, LONG MicroSeconds);
ERROR     WatchPublicMemory(APTR Memory, MEMORYID MemoryID, LONG MilliSeconds);

#define DelayMsg(a,b,c)     (ActionMsg(a,b,c,0,-1))
#define DelayAction(a,b,c)  (ActionMsg(a,b,c,0,-1))
#define SendAction(a,b,c,d) (ActionMsg(a,b,c,d,0))
#define WaitMsg(a,b,c)      (ActionMsg(a,b,c,0,-2))

#else

#define AccessMemory(a,b,c)       (KernelBase->AccessMemory(a,b,c))
#define AccessSemaphore(a,b,c)    (KernelBase->AccessSemaphore(a,b,c))
#define Action(a,b,c)             (KernelBase->Action(a,b,c))
#define ActionList()              (KernelBase->ActionList())
#define ActionMsg(a,b,c)          (KernelBase->_ActionMsg(a,b,c,0,0))
#define ActionMsgPort(a,b,c,d,e)  (KernelBase->_ActionMsg(a,b,c,d,e))
#define ActionTags                (KernelBase->ActionTag)
#define AddMsgHandler(a)          (KernelBase->AddMsgHandler(a))
#define AllocateClassID(a,b)      (KernelBase->AllocateClassID(a,b))
#define AllocateID(a)             (KernelBase->AllocateID(a))
#define AllocMemory(a,b,c,d)      (KernelBase->AllocMemory(a,b,c,d))
#define AllocSemaphore(a,b,c,d)   (KernelBase->AllocSemaphore(a,b,c,d))
#define AlterMemory(a,b)          (KernelBase->AlterMemory(a,b))
#define AccessObject(a,b,c)       (KernelBase->AccessObject(a,b,c))
#define CheckAction(a,b)          (KernelBase->CheckAction(a,b))
#define CheckMemoryExists(a)      (KernelBase->CheckMemoryExists(a))
#define CheckObjectExists(a,b)    (KernelBase->CheckObjectExists(a,b))
#define ClearMemory(a,b)          (KernelBase->ClearMemory(a,b))
#define CloneMemory(a,b,c,d)      (KernelBase->CloneMemory(a,b,c,d))
#define CopyMemory(a,b,c)         (KernelBase->CopyMemory(a,b,c))
#define CreateObject              (KernelBase->CreateObjectTags)
#define CurrentContext()          (KernelBase->CurrentContext())
#define CurrentTaskID()           ((OBJECTID)(KernelBase->_GetResource(RES_CURRENTTASK)))
#define CurrentTask()             ((OBJECTPTR)(LONG)(KernelBase->_GetResource(RES_CURRENTTASKPTR)))
#define DebugState(a,b)           (KernelBase->DebugState(a,b))
#define DeregisterFD(a)           (KernelBase->DeregisterFD(a))
#define DelayAction(a,b,c)        (KernelBase->_ActionMsg(a,b,c,0,-1))
#define DelayMsg(a,b,c)           (KernelBase->_ActionMsg(a,b,c,0,-1))
#define DeleteMsg(a,b)            (KernelBase->_UpdateMessage(a,b,-1,0,0))
#define DPrintF                   (KernelBase->DPrintFTags)
#define FindClass(a,b)            (KernelBase->FindClass(a,b))
#define FastFindObject(a,b,c,d,e) (KernelBase->FastFindObject(a,b,c,d,e))
#define FindObject(a,b,c,d)       (KernelBase->FindObject(a,b,c,d))
#define FindPrivateObject(a,b)    (KernelBase->FindPrivateObject(a,b))
#define Forbid()                  (KernelBase->Forbid())
#define FreeMemory(a)             (KernelBase->FreeMemory(a))
#define FreeMemoryID(a)           (KernelBase->FreeMemoryID(a))
#define FreeSemaphore(a)          (KernelBase->FreeSemaphore(a))
#define GetClassID(a)             (KernelBase->GetClassID(a))
#define GetOwnerID(a)             (KernelBase->GetOwnerID(a))
#define GetField(a,b,c,d)         (KernelBase->GetField(a,b,c,d))
#define GetFields                 (KernelBase->GetFieldTags)
#define GetMessage(a,b,c,d,e)     (KernelBase->GetMessage(a,b,c,d,e))
#define GetName(a,b,c)            (KernelBase->GetName(a,b,c))
#define GetFieldVariable(a,b,c,d) (KernelBase->GetFieldVariable(a,b,c,d))
#define GetResource(a)            (KernelBase->_GetResource(a))
#define ListChildren(a,b,c)       (KernelBase->ListChildren(a,b,c))
#define ListMemory(a,b)           (KernelBase->ListMemory(a,b))
#define ListObjects(a,b,c)        (KernelBase->ListObjects(a,b,c))
#define ListTasks(a,b)            (KernelBase->ListTasks(a,b))
#define ManageAction(a,b)         (KernelBase->ManageAction(a,b))
#define MemoryIDInfo(a,b,c)       (KernelBase->MemoryIDInfo(a,b,c))
#define MemoryPtrInfo(a,b,c)      (KernelBase->MemoryPtrInfo(a,b,c))
#define MessageHook(a,b)          (KernelBase->MessageHook(a,b))
#define NewClass(a,b)             (KernelBase->NewClass(a,b))
#define NewObject(a,b,c,d)        (KernelBase->_NewObject(a,b,c,d,0))
#define NotifySubscribers(a,b,c,d) (KernelBase->NotifySubscribers(a,b,c,d))
#define ObjectError(a,b,c)        (KernelBase->ObjectError(a,b,c))
#define ObtainMethod(a,b,c)       (KernelBase->ObtainMethod(a,b,c))
#define Permit()                  (KernelBase->Permit())
#define PrintDiagnosis()          (KernelBase->PrintDiagnosis(NULL,NULL))
#define ProcessMessages(a,b)      (KernelBase->ProcessMessages(a,b))
#define RandomNumber(a)           (KernelBase->RandomNumber(a))
#define ReallocMemory(a,b,c,d)    (KernelBase->ReallocMemory(a,b,c,d))
#define RegisterFD(a,b,c)         (KernelBase->RegisterFD(a,b,c))
#define ReleaseLock(a,b)          (KernelBase->ReleaseLock(a,b))
#define ReleaseMemory(a)          (KernelBase->ReleaseMemory(a))
#define ReleaseObject(a)          (KernelBase->ReleaseObject(a))
#define ReleaseSemaphore(a,b)     (KernelBase->ReleaseSemaphore(a,b))
#define RemoveMsgHandler(a)       (KernelBase->RemoveMsgHandler(a))
#define ResolveClassName(a)       (KernelBase->ResolveClassName(a))
#define ResolveFields             (KernelBase->ResolveFieldTags)
#define ScanMessages(a,b,c,d,e)   (KernelBase->ScanMessages(a,b,c,d,e))
#define SelfDestruct()            (KernelBase->SelfDestruct())
#define SendAction(a,b,c,d)       (KernelBase->_ActionMsg(a,b,c,d,0))
#define SendError(a,b)            (KernelBase->SendError(a,b))
#define SetContext(a)             (KernelBase->SetContext(a))
#define SetOwner(a,b)             (KernelBase->SetOwner(a,b))
#define SetField(a,b,c,d)         (KernelBase->SetField(a,b,c,d))
#define SetFields                 (KernelBase->SetFieldTags)
#define SetFieldVariable(a,b,c)   (KernelBase->SetFieldVariable(a,b,c))
#define SetName(a,b)              (KernelBase->SetName(a,b))
#define SetResource(a,b)          (KernelBase->SetResource(a,b))
#define StepBack()                (KernelBase->StepBack())
#define SendMessage(a,b,c,d,e)    (KernelBase->SendMessage(a,b,c,d,e))
#define StrCompare(a,b,c,d)       (KernelBase->StrCompare(a,b,c,d))
#define SubscribeAction(a,b,c)    (KernelBase->SubscribeAction(a,b,c))
#define SubscribeActionTags       (KernelBase->_SubscribeActionTags)
#define SubscribeChannel(a,b)     (KernelBase->SubscribeChannel(a,b))
#define SubscribeField(a,b,c)     (KernelBase->SubscribeField(a,b,c))
#define SubscribeTimer(a,b)       (KernelBase->SubscribeTimer(a,b))
#define SystemTime()              (KernelBase->SystemTime())
#define TotalChildren(a)          (KernelBase->TotalChildren(a))
#define UnsubscribeAction(a,b,c)  (KernelBase->UnsubscribeAction(a,b,c))
#define UnsubscribeChannel(a,b)   (KernelBase->UnsubscribeChannel(a,b))
#define UnsubscribeField(a,b,c)   (KernelBase->UnsubscribeField(a,b,c))
#define UnsubscribeTimer(a)       (KernelBase->UnsubscribeTimer(a))
#define UpdateMessage(a,b,c,d,e)  (KernelBase->_UpdateMessage(a,b,c,d,e))
#define WaitMsg(a,b,c)            (KernelBase->_ActionMsg(a,b,c,0,-2))
#define WaitTime(a,b)             (KernelBase->WaitTime(a,b))
#define WatchPublicMemory(a,b,c)  (KernelBase->WatchPublicMemory(a,b,c))
#endif /* LOCAL_KERNEL */

/*****************************************************************************
** Macros
*/

#define AttemptExclusive          AccessObject
#define FreeExclusive             ReleaseObject
#define CheckObjectIDExists(a)    (CheckObjectExists(a,0))
#define CheckObjectNameExists(a)  (CheckObjectExists(0,a))
#define GetOwner(a)               (((OBJECTPTR)(a))->OwnerID)
#define GetUniqueID(a)            (((OBJECTPTR)(a))->UniqueID)
#define AllocPublicMemory(a,b,c)  (AllocMemory((a),(b)|MEM_PUBLIC,0,(c)))
#define AllocPrivateMemory(a,b,c) (AllocMemory((a),(b),(c),0))
#define NewPrivateObject(a,b,c)   (NewObject(a,b,c,0))
#define NewPublicObject(a,b,c)    (NewObject(a,(b)|NF_PUBLIC,0,c))
#define NewChildObject(a,b,c)     (NewObject(a,(b)|NF_CHILD,0,c))
#define NewLocalObject(a,b,c)     (NewObject(a,(b)|NF_CHILD,0,c))
#define NewNamedObject(a,b,c,d,e) (KernelBase->_NewObject(a,(b)|NF_NAME,c,d,e))
#define StepObjectError(a,b,c)    (ObjectError(a,b,-c))

/****************************************************************************/

#ifndef MODULES_KERNELBASE_H
#include <pandora/modules/kernelbase.h>
#endif

#endif /* MODULES_KERNEL_H */
